#!/usr/bin/python3
"""
Copyright (C) 2012, 2013 Eduardo Naufel Schettino and Johan Mattsson

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program. If not, see <http://www.gnu.org/licenses/>.
"""

from . import version

VERSION = version.VERSION

def write_config (prefix):
    print ("Writing Config.vala")

    vars = (('VERSION', VERSION),
        ('PREFIX', prefix),
        )

    f = open('./libbirdfont/Config.vala', 'w+')
    f.write("// Don't edit this file -- it is generated by the build script\n")
    f.write("namespace BirdFont {\n")

    var_line = '    internal static const string %s = "%s";\n'
    for name, value in vars:
        f.write(var_line % (name, value))

    f.write("}")

def write_compile_parameters (prefix, dest, cc, gee, valac, non_null,
                              valacflags, cflags, ldflags, gtk):
    f = open('./scripts/config.py', 'w+')
    f.write("#!/usr/bin/python3\n")
    f.write("PREFIX =  \"" + prefix + "\"\n")
    f.write("DEST = \"" + dest + "\"\n")
    f.write("CC = \"" + cc + "\"\n")
    f.write("GEE = \"" + gee + "\"\n")
    f.write("VALAC = \"" + valac + "\"\n")

    if non_null:
        f.write("NON_NULL = \"--enable-experimental-non-null\"\n")
    else:
        f.write("NON_NULL = \"\"\n")
        
    f.write("VALACFLAGS = " + str(valacflags) + "\n")
    f.write("CFLAGS = " + str(cflags) + "\n")
    f.write("LDFLAGS = " + str(ldflags) + "\n")
    f.write("GTK = " + str(gtk) + "\n")



#!/usr/bin/python3
""" Copyright (C) 2013 2014 2015 Johan Mattsson

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""

import os
import subprocess
import glob
import platform
import sys
from optparse import OptionParser
from scripts import config
from scripts import version
from scripts.run import run

def getDest (file, dir):
	f = dest + prefix + dir + '/'
	s = file.rfind ('/')
	if s > -1:
		f += file[s + 1:]
	else:
		f += file
	return f

def getDestRoot (file, dir):
	f = dest + dir + '/'
	s = file.rfind ('/')
	if s > -1:
		f += file[s + 1:]
	else:
		f += file
	return f
	
def install (file, dir, mode):
	f = getDest (file, dir)
	print ("install: " + f)
	run ('install -d ' + dest + prefix + dir)
	run ('install -m ' + str(mode) + ' ' + file + ' ' + dest + prefix + dir + '/')
	installed.write (f + "\n")

def install_root (file, dir, mode):
	f = getDestRoot (file, dir)
	print ("install: " + f)
	run ('install -d ' + dest + dir)
	run ('install -m ' + str(mode) + ' ' + file + ' ' + dest + dir + '/')

def link (dir, file, linkname):
	f = getDest (linkname, dir)
	print ("install link: " + f)
	run ('cd ' + dest + prefix + dir + ' && ln -sf ' + file + ' ' + linkname)
	installed.write (f + "\n")
	
if not os.path.exists ("build/configured"):
	print ("Project is not configured")
	exit (1)

parser = OptionParser()
parser.add_option ("-d", "--dest", dest="dest", help="install to this directory", metavar="DEST")
parser.add_option ("-m", "--nogzip", dest="nogzip", help="don't gzip manpages", default=False)
parser.add_option ("-n", "--manpages-directory", dest="mandir", help="put man pages in this directory under prefix")
parser.add_option ("-l", "--libdir", dest="libdir", help="path to directory for shared libraries (lib or lib64).")
parser.add_option ("-c", "--skip-command-line-tools", dest="nocli", help="don't install command line tools")
parser.add_option ("-a", "--apport", dest="apport", help="install apport scripts", default=True)

(options, args) = parser.parse_args()

if not options.dest:
	options.dest = ""

if not options.nocli:
	options.nocli = False

nogzip = options.nogzip

if not options.mandir:
	mandir = "/man/man1"
else: 
	mandir = options.mandir

prefix = config.PREFIX
dest = options.dest

# create uninstall file
installed = open ('build/installed', 'w')
installed.write ('build/installed\n')

# install it:
install ('resources/icons.bf', '/share/birdfont', 644)
install ('resources/bright.theme', '/share/birdfont', 644)
install ('resources/dark.theme', '/share/birdfont', 644)
install ('resources/high_contrast.theme', '/share/birdfont', 644)
install ('resources/key_bindings.xml', '/share/birdfont', 644)
install ('resources/linux/birdfont_window_icon.png', '/share/birdfont', 644)
install ('resources/linux/birdfont.desktop', '/share/applications', 644)
install ('resources/ucd.sqlite', '/share/birdfont', 644)
install ('resources/codepages.sqlite', '/share/birdfont', 644)
install ('resources/Roboto-Regular.ttf', '/share/birdfont', 644)

install ('resources/linux/256x256/birdfont.png', '/share/icons/hicolor/256x256/apps', 644)
install ('resources/linux/128x128/birdfont.png', '/share/icons/hicolor/128x128/apps', 644)
install ('resources/linux/48x48/birdfont.png', '/share/icons/hicolor/48x48/apps', 644)

install ('resources/linux/birdfont.appdata.xml', '/share/appdata', 644)

if os.path.isfile ('build/bin/birdfont'):
	install ('build/bin/birdfont', '/bin', 755)

if not options.nocli:
	install ('build/bin/birdfont-autotrace', '/bin', 755)
	install ('build/bin/birdfont-export', '/bin', 755)
	install ('build/bin/birdfont-import', '/bin', 755)

#library
if sys.platform == 'darwin':
	libdir = '/lib'
elif not options.libdir:
	
	if platform.dist()[0] == 'Ubuntu' or platform.dist()[0] == 'Debian':
		process = subprocess.Popen(['dpkg-architecture', '-qDEB_HOST_MULTIARCH'], stdout=subprocess.PIPE)
		out, err = process.communicate()
		out = out.decode('ascii')
		libdir = '/lib/' + out.rstrip ('\n')
	else:
		p = platform.machine()
		if p == 'i386' or p == 's390' or p == 'ppc' or p == 'armv7hl':
			libdir = '/lib'
		elif p == 'x86_64' or p == 's390x' or p == 'ppc64':
			libdir = '/lib64'
		else:
			libdir = '/lib'
else:
	libdir = options.libdir

if "openbsd" in sys.platform:
        install ('build/bin/libbirdfont.so.' + '${LIBbirdfont_VERSION}', '/lib', 644)
elif os.path.isfile ('build/bin/libbirdfont.so.' + version.SO_VERSION):
	install ('build/bin/libbirdfont.so.' + version.SO_VERSION, libdir, 644)
	link (libdir, 'libbirdfont.so.' + version.SO_VERSION, ' libbirdfont.so.' + version.SO_VERSION_MAJOR)
	link (libdir, 'libbirdfont.so.' + version.SO_VERSION, ' libbirdfont.so')
elif os.path.isfile ('build/libbirdfont.so.' + version.SO_VERSION):
	install ('build/libbirdfont.so.' + version.SO_VERSION, libdir, 644)
	link (libdir, 'libbirdfont.so.' + version.SO_VERSION, ' libbirdfont.so.' + version.SO_VERSION_MAJOR)
	link (libdir, 'libbirdfont.so.' + version.SO_VERSION, ' libbirdfont.so')
elif os.path.isfile ('build/bin/libbirdfont.' + version.SO_VERSION + '.dylib'):
	install ('build/bin/libbirdfont.' + version.SO_VERSION + '.dylib', libdir, 644)
	link (libdir, 'libbirdfont.' + version.SO_VERSION + '.dylib', ' libbirdfont.dylib.' + version.SO_VERSION_MAJOR)
	link (libdir, 'libbirdfont.' + version.SO_VERSION + '.dylib', ' libbirdfont.dylib')
else:
	print ("Can't find libbirdfont.")
	exit (1)

if "openbsd" in sys.platform:
        install ('build/bin/libbirdgems.so.' + '${LIBbirdgems_VERSION}', '/lib', 644)
elif os.path.isfile ('build/bin/libbirdgems.so.' + version.LIBBIRDGEMS_SO_VERSION):
        install ('build/bin/libbirdgems.so.' + version.LIBBIRDGEMS_SO_VERSION, libdir, 644)
        link (libdir, 'libbirdgems.so.' + version.LIBBIRDGEMS_SO_VERSION, ' libbirdgems.so.' + version.LIBBIRDGEMS_SO_VERSION_MAJOR)
        link (libdir, 'libbirdgems.so.' + version.LIBBIRDGEMS_SO_VERSION, ' libbirdgems.so')
elif os.path.isfile ('build/libbirdgems.so.' + version.LIBBIRDGEMS_SO_VERSION):
        install ('build/libbirdgems.so.' + version.LIBBIRDGEMS_SO_VERSION, libdir, 644)
        link (libdir, 'libbirdgems.so.' + version.LIBBIRDGEMS_SO_VERSION, ' libbirdgems.so.' + version.LIBBIRDGEMS_SO_VERSION_MAJOR)
        link (libdir, 'libbirdgems.so.' + version.LIBBIRDGEMS_SO_VERSION, ' libbirdgems.so')
elif os.path.isfile ('build/bin/libbirdgems.' + version.LIBBIRDGEMS_SO_VERSION + '.dylib'):
        install ('build/bin/libbirdgems.' + version.LIBBIRDGEMS_SO_VERSION + '.dylib', libdir, 644)
        link (libdir, 'libbirdgems.' + version.LIBBIRDGEMS_SO_VERSION + '.dylib', ' libbirdgems.dylib.' + version.LIBBIRDGEMS_SO_VERSION_MAJOR)
        link (libdir, 'libbirdgems.' + version.LIBBIRDGEMS_SO_VERSION + '.dylib', ' libbirdgems.dylib')
else:
	print ("Can't find libbirdgems, version: " + version.LIBBIRDGEMS_SO_VERSION)
	exit (1)

	
#manpages

if not nogzip:
    install ('build/birdfont.1.gz', mandir, 644)

    if not options.nocli:
        install ('build/birdfont-autotrace.1.gz', mandir, 644)
        install ('build/birdfont-export.1.gz', mandir, 644)
        install ('build/birdfont-import.1.gz', mandir, 644)
else:
    install ('resources/linux/birdfont.1', mandir, 644)

    if not options.nocli:
        install ('resources/linux/birdfont-autotrace.1', mandir, 644)
        install ('resources/linux/birdfont-export.1', mandir, 644)
        install ('resources/linux/birdfont-import.1', mandir, 644)

#translations
for lang_dir in glob.glob('build/locale/*'):
	lc = lang_dir.replace ('build/locale/', "")
	install ('build/locale/' + lc + '/LC_MESSAGES/birdfont.mo', '/share/locale/' + lc + '/LC_MESSAGES' , 644);

#file type 
install ('resources/linux/birdfont.xml', '/share/mime/packages', 644)

#apport hooks
if options.apport == "True":
	install ('resources/birdfont.py', '/share/apport/package-hooks', 644)
	install ('resources/source_birdfont.py', '/share/apport/package-hooks', 644)
	install_root ('resources/birdfont-crashdb.conf', '/etc/apport/crashdb.conf.d', 644)

installed.close ()



import os
import fnmatch
import glob
import types
import time
from os import path
from scripts.run import run

def get_sources_path(directory, pattern):
    """obtain path of all source files for a matching pattern"""
    matches = []
    for root, dirnames, filenames in os.walk(directory):
        for filename in filenames:
            full_path = os.path.join(root, filename)
            if fnmatch.filter([full_path], pattern):
                matches.append(os.path.join(root, filename))
    return matches
    
class Builder(object):
    """helper to generate tasks to compile vala code"""

    def __init__(self,
                 source_directory,
                 valac_command,
                 cc_command,
                 linker_command,
                 target_binary,
                 link = None,
                 dependencies = None):
        self.source_directory = source_directory
        self.valac_command = valac_command
        self.cc_command = cc_command
        self.linker_command = linker_command
        self.target_binary = target_binary
        self.link = link
        self.dependencies = dependencies

    def build(self):
        source_directory = self.source_directory
        valac_command = self.valac_command
        cc_command = self.cc_command
        linker_command = self.linker_command
        target_binary = self.target_binary
        
        if not self.dependencies == None:
            bindep = [path.join('build', 'bin', f) for f in self.dependencies]
        else:
            bindep = []
            
        copied_csource_paths = get_sources_path(source_directory, '*.c')
        copied_cheader_paths = get_sources_path(source_directory, '*.h')
        vala_source_paths = get_sources_path(source_directory, '*.vala')
        build_directory = path.join('build', source_directory)

        csource_files = [path.basename(f) for f in copied_csource_paths]
        generated_csource_paths = []
        for vala_path in vala_source_paths:
            vala_file = path.basename(vala_path)
            cfile = vala_file.replace('.vala', '.c')
            cpath = path.join(build_directory, cfile)
            generated_csource_paths.append(cpath)
            csource_files.append(cfile)
       
        build_file = path.join(build_directory, 'placeholder')
        yield {
            'basename': 'mkdir ' + build_directory,
            'actions': ['mkdir -p ' + path.join('build', 'bin'),
                        'mkdir -p ' + build_directory, 
                        '[ -e "' + build_file + '" ] || touch "' + build_file + '"'],
            'targets': [build_file],
        }
        
        copied_csources = []
        copied_cheader = [] 
        for csource in copied_cheader_paths + copied_csource_paths:
            dest = path.join(build_directory, path.basename(csource))
            
            if not dest[-2:] == ".h":
                copied_csources.append(dest)
            else:
                copied_cheader.append(dest)
                
            yield {
                'basename': 'copy ' + csource,
                'file_dep': [build_file] + [csource],
                'actions': ['cp ' + csource + ' ' + build_directory],
                'targets': [dest]
            }
        
        yield {
            'basename': 'valac ' + source_directory,
            'file_dep': [build_file] + vala_source_paths + bindep,
            'actions': [valac_command],
            'targets': generated_csource_paths
        }
               
        csource_paths = generated_csource_paths + copied_csources
        object_files = []
        
        for csource in csource_paths:
            object_file = path.basename (csource.replace('.c', '.o'))
            object_files.append(object_file);
             
            command = cc_command.replace('C_SOURCE', csource)
            object_path = path.join(build_directory, object_file)
            command = command.replace('OBJECT_FILE', object_path)
            yield {
                'basename': 'compile ' + csource,
                'file_dep': [build_file, csource] + bindep + copied_cheader,
                'actions': [command],
                'targets': [path.join(build_directory, object_file)],
            }
            
        object_paths = [path.join(build_directory, f) for f in object_files] 
        yield {
            'basename': source_directory,
            'file_dep': object_paths + [build_file] + bindep,
            'actions': [linker_command],
            'targets': [path.join('build', 'bin', target_binary)]
        }

        if not self.link == None:
            createlink = 'cd build/bin/ && ln -s -f ' + target_binary + ' ' + self.link
            yield {
                'basename': 'Create link ' + target_binary + ' ' + self.link,
                'file_dep': [path.join('build', 'bin', target_binary)],
                'actions': [createlink],
                'targets': [path.join('build', 'bin', self.link)]
            }

def is_up_to_date(task):
    for target in task['targets']:
        if not path.isfile(target):
            return False

    if not 'file_dep' in task.keys():
        return False

    for dep in task['file_dep']:
        if not path.isfile(dep):
            print('Dependency is not created yet: ' + dep + ' needed for ' + task['targets'])
            exit(1)

    target_times = []
    for target in task['targets']:
        target_times.append(path.getmtime(target))
    target_times.sort()

    dependency_times = []
    for dependency in task['file_dep']:
        if not path.basename(dependency) == 'placeholder':
            dependency_times.append(path.getmtime(dependency))	
    dependency_times.sort()

    if len(dependency_times) == 0 or len(target_times) == 0:
        return False

    return dependency_times[-1] <= target_times[0]


def get_name(task):
    try:
	    return task['name']
    except KeyError:
        return task['basename']

def execute_task(task):
    if is_up_to_date(task):
        print(get_name(task) + ' - up to date.')
    else:
        for action in task['actions']:
            print(action)
            run(action)

def process_tasks(generator):
	for task in generator:
		if isinstance(task, types.GeneratorType):
			process_tasks(task)
		else:
			execute_task(task)




#!/usr/bin/python3

import subprocess
from os import path

from run import run

def fuzz_import():
    fuzz_bf ('birdfont-test/testfont.bf')

def fuzz_bf (file):
    run ("mkdir -p build/fuzz")
    run ("mkdir -p build/fuzz/bugs")
    run ("radamsa " + file + " > build/fuzz/font.bf")

    cmd = "./birdfont-test.sh BF build/fuzz/font.bf"
    print('Running: ' + cmd)
    process = subprocess.Popen (cmd, shell=True)
    process.communicate()[0]
    if not process.returncode == 0:
        print("Error: " + cmd)
        print("A bug was found.")
        
        i = 0
        while path.isfile ('build/bugs/font_' + str(i) + '.bf'):
            i = i + 1
			
        run ('mv build/fuzz/font.bf attic/fuzz/bugs/font_' + str(i) + '.bf')

    run ("rm -f build/fuzz/font.svg")

while True:
    fuzz_import ()



"""
Copyright (C) 2012 2013 2014 2015 Eduardo Naufel Schettino and Johan Mattsson

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program. If not, see <http://www.gnu.org/licenses/>.
"""

import os
import glob
import subprocess
import sys

from scripts import version
from scripts.translations import compile_translations
from scripts import config
from scripts.builder import Builder

DOIT_CONFIG = {
    'default_tasks': [
        'build',
        'compile_translations',
        'man',
        'libbirdfont', 
        'libbirdgems', 
        'birdfont', 
        'birdfont-autotrace',
        'birdfont-export',
        'birdfont-import',
        'birdfont-test'
        ],
    }

if "kfreebsd" in sys.platform:
    LIBBIRDGEMS_SO_VERSION=version.LIBBIRDGEMS_SO_VERSION
elif "openbsd" in sys.platform:
    LIBBIRDGEMS_SO_VERSION='${LIBbirdgems_VERSION}'
else:
    LIBBIRDGEMS_SO_VERSION=version.LIBBIRDGEMS_SO_VERSION

if "kfreebsd" in sys.platform:
    SO_VERSION=version.SO_VERSION
elif "openbsd" in sys.platform:
    SO_VERSION='${LIBbirdfont_VERSION}'
else:
    SO_VERSION=version.SO_VERSION

def soname(target_binary):
    if "darwin" in sys.platform or "msys" in sys.platform:
        return ''
        
    return '-Wl,-soname,' + target_binary

def make_birdfont(target_binary, deps):
    valac_command = config.VALAC + """\
        -C \
        --vapidir=./ \
        --basedir build/birdfont/ \
        """ + config.NON_NULL + """ \
        """ + config.VALACFLAGS.get("birdfont", "") + """ \
        --enable-experimental \
        birdfont/*.vala \
		--vapidir=./ \
		--pkg """ + config.GEE + """ \
		--pkg gio-2.0  \
		--pkg cairo \
		--pkg libsoup-2.4 \
		--pkg gdk-pixbuf-2.0 \
		--pkg webkit2gtk-4.0 \
		--pkg libnotify \
		--pkg xmlbird \
		--pkg libbirdfont
        """
        
    cc_command = config.CC + " " + config.CFLAGS.get("birdfont", "") + """ \
        -c C_SOURCE \
		-D 'GETTEXT_PACKAGE="birdfont"' \
        -I./build/libbirdfont \
		$(pkg-config --cflags sqlite3) \
		$(pkg-config --cflags """ + config.GEE + """) \
		$(pkg-config --cflags gio-2.0) \
		$(pkg-config --cflags cairo) \
		$(pkg-config --cflags glib-2.0) \
		$(pkg-config --cflags gdk-pixbuf-2.0) \
		$(pkg-config --cflags webkit2gtk-4.0) \
		$(pkg-config --cflags libnotify) \
        -o OBJECT_FILE"""
        
    linker_command = config.CC + " " + config.LDFLAGS.get("birdfont", "") + """ \
        build/birdfont/*.o \
		-L./build/bin -lbirdfont \
		$(pkg-config --libs sqlite3) \
		$(pkg-config --libs """ + config.GEE + """) \
		$(pkg-config --libs gio-2.0) \
		$(pkg-config --libs cairo) \
		$(pkg-config --libs glib-2.0) \
		$(pkg-config --libs gdk-pixbuf-2.0) \
		$(pkg-config --libs webkit2gtk-4.0) \
		$(pkg-config --libs xmlbird) \
		$(pkg-config --libs libnotify) \
		-L./build -L./build/bin -l birdgems\
        -o build/bin/""" + target_binary

    birdfont = Builder('birdfont',
                          valac_command, 
                          cc_command,
                          linker_command,
                          target_binary,
                          None,
                          deps)
			
    yield birdfont.build()

def task_birdfont():
    yield make_birdfont('birdfont', ['libbirdgems.so', 'libbirdfont.so'])

def make_birdfont_export(target_binary, deps):
    valac_command = config.VALAC + """ \
        -C \
		--enable-experimental \
        --basedir build/birdfont-export/ \
        """ + config.NON_NULL + """ \
        """ + config.VALACFLAGS.get("birdfont-export", "") + """ \
		birdfont-export/*.vala \
		--vapidir=./ \
		--pkg """ + config.GEE + """ \
		--pkg gio-2.0  \
		--pkg cairo \
		--pkg xmlbird \
		--pkg libbirdfont
        """

    cc_command = config.CC + " " + config.CFLAGS.get("birdfont-export", "") + """ \
        -c C_SOURCE \
		-D 'GETTEXT_PACKAGE="birdfont"' \
        -I./build/libbirdfont \
		$(pkg-config --cflags sqlite3) \
		$(pkg-config --cflags """ + config.GEE + """) \
		$(pkg-config --cflags gio-2.0) \
		$(pkg-config --cflags cairo) \
		$(pkg-config --cflags glib-2.0) \
        -o OBJECT_FILE"""
        
    linker_command = config.CC + " " + config.LDFLAGS.get("birdfont-export", "") + """ \
		build/birdfont-export/*.o \
		-Lbuild/bin/ -lbirdfont \
		-lm \
		$(pkg-config --libs sqlite3) \
		$(pkg-config --libs """ + config.GEE + """) \
		$(pkg-config --libs gio-2.0) \
		$(pkg-config --libs cairo) \
		$(pkg-config --libs glib-2.0) \
		$(pkg-config --libs xmlbird) \
		-L./build -L./build/bin -l birdgems\
		-o ./build/bin/""" + target_binary

    birdfont_export = Builder('birdfont-export',
                              valac_command, 
                              cc_command,
                              linker_command,
                              target_binary,
                              None,
                              deps)
			
    yield birdfont_export.build()

def task_birdfont_export():
    yield make_birdfont_export('birdfont-export', ['libbirdgems.so', 'libbirdfont.so'])

def make_birdfont_import(target_binary, deps):
    valac_command = config.VALAC + """\
        -C  \
		--enable-experimental \
        --basedir build/birdfont-import/ \
        """ + config.NON_NULL + """ \
        """ + config.VALACFLAGS.get("birdfont-import", "") + """ \
		birdfont-import/*.vala \
		--vapidir=./ \
		--pkg """ + config.GEE + """ \
		--pkg gio-2.0  \
		--pkg cairo \
		--pkg xmlbird \
		--pkg libbirdfont
        """
        
    cc_command = config.CC + " " + config.CFLAGS.get("birdfont-import", "") + """ \
        -c C_SOURCE \
		-D 'GETTEXT_PACKAGE="birdfont"' \
        -I./build/libbirdfont \
		$(pkg-config --cflags sqlite3) \
		$(pkg-config --cflags """ + config.GEE + """) \
		$(pkg-config --cflags gio-2.0) \
		$(pkg-config --cflags cairo) \
		$(pkg-config --cflags glib-2.0) \
        -o OBJECT_FILE"""

    linker_command = config.CC + " " + config.LDFLAGS.get("birdfont-import", "") + """ \
		build/birdfont-import/*.o \
		-Lbuild/bin/ -lbirdfont \
		-lm \
		$(pkg-config --libs sqlite3) \
		$(pkg-config --libs """ + config.GEE + """) \
		$(pkg-config --libs gio-2.0) \
		$(pkg-config --libs cairo) \
		$(pkg-config --libs glib-2.0) \
		$(pkg-config --libs xmlbird) \
		-L./build -L./build/bin -l birdgems\
		-o ./build/bin/""" + target_binary

    birdfont_import = Builder('birdfont-import',
                          valac_command, 
                          cc_command,
                          linker_command,
                          target_binary,
                          None,
                          deps)
			
    yield birdfont_import.build()

def task_birdfont_import():
    yield make_birdfont_import('birdfont-import', ['libbirdgems.so', 'libbirdfont.so'])
	
def make_birdfont_autotrace(target_binary, deps):
    valac_command = config.VALAC + """\
        -C \
		--enable-experimental \
        --basedir build/birdfont-autotrace/ \
        """ + config.NON_NULL + """ \
        """ + config.VALACFLAGS.get("birdfont-autotrace", "") + """ \
		birdfont-autotrace/*.vala \
		--vapidir=./ \
		--pkg """ + config.GEE + """ \
		--pkg gio-2.0  \
		--pkg cairo \
		--pkg xmlbird \
		--pkg libbirdfont \
        """
        
    cc_command = config.CC + " " + config.CFLAGS.get("birdfont-autotrace", "") + """ \
        -c C_SOURCE \
		-D 'GETTEXT_PACKAGE="birdfont"' \
        -I./build/libbirdfont \
		$(pkg-config --cflags sqlite3) \
		$(pkg-config --cflags """ + config.GEE + """) \
		$(pkg-config --cflags gio-2.0) \
		$(pkg-config --cflags cairo) \
		$(pkg-config --cflags glib-2.0) \
        -o OBJECT_FILE"""
        
    linker_command = config.CC + " " + config.LDFLAGS.get("birdfont-autotrace", "") + """ \
		build/birdfont-autotrace/*.o \
        -I./build/libbirdfont \
		-Lbuild/bin/ -lbirdfont \
		-lm \
		$(pkg-config --libs sqlite3) \
		$(pkg-config --libs """ + config.GEE + """) \
		$(pkg-config --libs gio-2.0) \
		$(pkg-config --libs cairo) \
		$(pkg-config --libs glib-2.0) \
		$(pkg-config --libs xmlbird) \
		-L./build -L./build/bin -l birdgems\
		-o ./build/bin/""" + target_binary

    birdfont_autotrace = Builder('birdfont-autotrace',
                          valac_command, 
                          cc_command,
                          linker_command,
                          target_binary,
                          None,
                          deps)
			
    yield birdfont_autotrace.build()

def task_birdfont_autotrace():
    yield make_birdfont_autotrace('birdfont-autotrace', ['libbirdgems.so', 'libbirdfont.so'])
    
def make_libbirdfont(target_binary, deps):
    valac_command = config.VALAC + """\
        -C \
        --vapidir=./ \
        --basedir build/libbirdfont/ \
        """ + config.NON_NULL + """ \
        """ + config.VALACFLAGS.get("libbirdfont", "") + """ \
        --enable-experimental \
        --library libbirdfont \
        -H build/libbirdfont/birdfont.h \
        libbirdfont/*.vala \
        libbirdfont/OpenFontFormat/*.vala \
        libbirdfont/Renderer/*.vala \
        --pkg """ + config.GEE + """ \
        --pkg gio-2.0 \
        --pkg cairo \
        --pkg xmlbird \
        --pkg libbirdgems \
        --pkg sqlite3 \
        """

    cc_command = config.CC + " " + config.CFLAGS.get("libbirdfont", "") + """ \
            -c C_SOURCE \
            -fPIC \
            -D 'GETTEXT_PACKAGE="birdfont"' \
            -I ./build/libbirdfont \
            -I ./build/libbirdgems \
            $(pkg-config --cflags sqlite3) \
            $(pkg-config --cflags fontconfig) \
            $(pkg-config --cflags """ + config.GEE + """) \
            $(pkg-config --cflags gio-2.0) \
            $(pkg-config --cflags cairo) \
            $(pkg-config --cflags glib-2.0) \
            $(pkg-config --cflags xmlbird) \
            -o OBJECT_FILE"""

    linker_command = config.CC + " " + config.LDFLAGS.get("libbirdfont", "") + """ \
            -shared \
            """ + soname(target_binary) + """ \
            build/libbirdfont/*.o \
            $(pkg-config --libs sqlite3) \
            $(freetype-config --libs) \
            $(pkg-config --libs """ + config.GEE + """) \
            $(pkg-config --libs gio-2.0) \
            $(pkg-config --libs fontconfig) \
            $(pkg-config --libs cairo) \
            $(pkg-config --libs glib-2.0) \
            $(pkg-config --libs xmlbird) \
            -L./build -L./build/bin -l birdgems\
            -o ./build/bin/""" + target_binary

    libbirdfont = Builder('libbirdfont',
                          valac_command, 
                          cc_command,
                          linker_command,
                          target_binary,
                          'libbirdfont.so',
                          deps)
			
    yield libbirdfont.build()

def task_libbirdfont():
    yield make_libbirdfont('libbirdfont.so.' + SO_VERSION, ['libbirdgems.so'])
    
def make_libbirdgems(target_binary, deps):
    valac_command = config.VALAC + """\
		-C \
		-H build/libbirdgems/birdgems.h \
		--pkg posix \
		--vapidir=./ \
		--basedir=build/libbirdgems/ \
		""" + config.NON_NULL + """ \
        """ + config.VALACFLAGS.get("libbirdgems", "") + """ \
		--enable-experimental \
		--library libbirdgems \
		libbirdgems/*.vala \
        """

    cc_command = config.CC + " " + config.CFLAGS.get("libbirdgems", "") + """ \
			-fPIC \
			$(pkg-config --cflags glib-2.0) \
			-c C_SOURCE \
            -o OBJECT_FILE \
			"""

    linker_command = config.CC + " " + config.LDFLAGS.get("libbirdgems", "") + """ \
			-shared \
			""" + soname(target_binary) + """ \
			-fPIC \
			build/libbirdgems/*.o \
			$(pkg-config --libs glib-2.0) \
			$(pkg-config --libs gobject-2.0) \
			-o build/bin/""" + target_binary

    libbirdgems = Builder('libbirdgems',
                          valac_command, 
                          cc_command,
                          linker_command,
                          target_binary,
						  'libbirdgems.so',
                          deps)
			
    yield libbirdgems.build()

def task_libbirdgems():
    yield make_libbirdgems('libbirdgems.so.' + LIBBIRDGEMS_SO_VERSION, []) 

def task_compile_translations ():
    """translate po files"""
    return  {
        'actions': [compile_translations]
        }
        
def task_man():
    """gzip linux man pages"""
    for name in ("birdfont.1", "birdfont-export.1", 
                 "birdfont-import.1", "birdfont-autotrace.1"):
        yield {
            'name': name,
            'file_dep': ['resources/linux/' + name],
            'targets': ['build/' + name + '.gz'],
            'actions': ['gzip -9 -c resources/linux/' + name + ' > ' + 'build/' + name + '.gz'],
            }

def task_distclean ():
    return  {
        'actions': ['rm -rf .doit.db build scripts/config.py'
                    + ' libbirdfont/Config.vala'
                    + ' __pycache__ scripts/__pycache__']
        }

def task_build ():
    if not os.path.exists ("build/configured"):
        print ("Project is not configured")
        exit (1)

    subprocess.check_output ('mkdir -p build', shell=True)
    subprocess.check_output ('touch build/installed', shell=True)

    return  {
        'actions': ['echo "Build"'],
        }

def make_birdfont_test(target_binary, deps):
    valac_command = config.VALAC + """\
        -C \
        --vapidir=./ \
        --basedir build/birdfont-test/ \
        """ + config.NON_NULL + """ \
        """ + config.VALACFLAGS.get("birdfont-test", "") + """ \
        --enable-experimental \
        birdfont-test/*.vala \
		--vapidir=./ \
		--pkg """ + config.GEE + """ \
		--pkg gio-2.0  \
		--pkg cairo \
		--pkg xmlbird \
		--pkg libbirdfont
        """

    cc_command = config.CC + " " + config.CFLAGS.get("birdfont-test", "") + """ \
        -c C_SOURCE \
		-D 'GETTEXT_PACKAGE="birdfont"' \
        -I./build/libbirdfont \
		$(pkg-config --cflags sqlite3) \
		$(pkg-config --cflags """ + config.GEE + """) \
		$(pkg-config --cflags gio-2.0) \
		$(pkg-config --cflags cairo) \
		$(pkg-config --cflags glib-2.0) \
		$(pkg-config --cflags xmlbird) \
        -o OBJECT_FILE"""
        
    linker_command = config.CC + " " + config.LDFLAGS.get("birdfont-test", "") + """ \
        build/birdfont-test/*.o \
		-L./build/bin -lbirdfont \
		$(pkg-config --libs sqlite3) \
		$(pkg-config --libs """ + config.GEE + """) \
		$(pkg-config --libs gio-2.0) \
		$(pkg-config --libs cairo) \
		$(pkg-config --libs glib-2.0) \
		$(pkg-config --libs xmlbird) \
		-L./build -L./build/bin -l birdgems\
        -o build/bin/""" + target_binary

    test = Builder('birdfont-test',
                   valac_command, 
                   cc_command,
                   linker_command,
                   target_binary,
                   None,
                   deps)
			
    yield test.build()

def task_birdfont_test():
    yield make_birdfont_test('birdfont-test', ['libbirdgems.so', 'libbirdfont.so'])





#!/usr/bin/python3
import version
from optparse import OptionParser

parser = OptionParser()
parser.add_option('-f', '--free', dest='free', action="store_true", help="generate non-commercial version")
(options, args) = parser.parse_args()

template = open('./scripts/snapcraft.yaml', 'r')
yaml = template.read();

if options.free:
	v = version.VERSION + "-free"
else:
	v = version.VERSION

yaml = yaml.replace("VERSION", v)

make_script = open('./build/snapsource/birdfont/Makefile', 'w+')
make_script.write("#Don't edit, the file is generated by a script\n")
make_script.write("all:\n")

if options.free:
    make_script.write ("\t./configure --valac-flags=\"-D FREE\"\n")
else:
    make_script.write ("\t./configure\n")

make_script.write("\t./build.py\n")
make_script.write("""

install:
	./install.py -d $(DESTDIR)
""")

build_script = open('./build/snapcraft.yaml', 'w+')
build_script.write(yaml)

